#include <ansi_c.h>
#include "rspr.h"
#include <cvirte.h>		
#include <userint.h>
#include "rspr_cvi_FSCAN_example.h"

#define CHECKERR(fcal) if ((rspr_status = fcal) < 0)\
                       {\
                            cleanup(VI_FALSE);\
                            return rspr_status;\
                       }
#define PORT 50000
#define RSPR_RING_BUFFER_SIZE 60000
#define RSPR_READ_BUFFER_SIZE 50

static int panelHandle;
ViSession io = 0;
ViStatus  rspr_status = 0;
ViBoolean gRun = VI_FALSE;
ViUInt32 gFreqHigh[RSPR_READ_BUFFER_SIZE];
ViUInt32 gFreqLow[RSPR_READ_BUFFER_SIZE];
ViInt16 gLevel[RSPR_READ_BUFFER_SIZE];
ViReal64 gFrequency[RSPR_READ_BUFFER_SIZE];

void cleanup (ViBoolean deviceClear);

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "rspr_cvi_FSCAN_example.uir", PANEL)) < 0)
		return -1;
	DisplayPanel (panelHandle);
    SuspendTimerCallbacks ();
	RunUserInterface ();
	DiscardPanel (panelHandle);
	return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
    ViChar      instrDescr[128];
    ViReal64    start, stop, time, rbw, stepsize, bw;
    ViInt32     time_mode, demod, meas_mode, level;
    
    switch (event)
    {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, PANEL_START, ATTR_DIMMED, 1);
            SetWaitCursor(1);
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instrDescr);
            GetCtrlVal (panelHandle, PANEL_START_FREQ, &start);
            GetCtrlVal (panelHandle, PANEL_STOP_FREQ, &stop);
            GetCtrlVal (panelHandle, PANEL_TIME_MODE, &time_mode);
            GetCtrlVal (panelHandle, PANEL_MEAS_TIME, &time);
            GetCtrlVal (panelHandle, PANEL_RBW, &rbw);
            GetCtrlVal (panelHandle, PANEL_STEPSIZE, &stepsize);
            GetCtrlVal (panelHandle, PANEL_BW, &bw);
            GetCtrlVal (panelHandle, PANEL_DEMODULATOR, &demod);
            GetCtrlVal (panelHandle, PANEL_MEAS_MODE, &meas_mode);
            GetCtrlVal (panelHandle, PANEL_LEVEL, &level);
            
            /* Open the session */
            if (io == 0)
            {
                CHECKERR (rspr_init (instrDescr, VI_TRUE, VI_TRUE, &io));
            }
            CHECKERR (rspr_ConfigureUDPStreamRingBufferSize(io,RSPR_RING_BUFFER_SIZE));
            /* Remove all previous stream definitions */
            CHECKERR (rspr_DeleteUDPStreamAddress (io, VI_TRUE,"",0));
            /* Establish a stream from instrument to this computer */
            CHECKERR (rspr_ConfigureLocalUDPStream (io, PORT, VI_TRUE, RSPR_VAL_ADD,
                                                    RSPR_VAL_TAG_FSCAN, RSPR_VAL_FLAG_LEVEL));
            CHECKERR (rspr_ConfigureLocalUDPStream (io, PORT, VI_TRUE, RSPR_VAL_ADD,
                                                    RSPR_VAL_TAG_FSCAN,
                                                    RSPR_VAL_FLAG_FREQ_LOW));
            CHECKERR (rspr_ConfigureLocalUDPStream (io, PORT, VI_TRUE, RSPR_VAL_ADD,
                                                    RSPR_VAL_TAG_FSCAN,
                                                    RSPR_VAL_FLAG_FREQ_HIGH));

            /* Configure measurement parameters */
            CHECKERR (rspr_ConfigureFrequencyScan (io, start, stop, stepsize, rbw));
            CHECKERR (rspr_ConfigureRXDemodulation (io, demod));
            CHECKERR (rspr_ConfigureRXBandwidth (io, bw));
            CHECKERR (rspr_ConfigureRXMeasure (io, level, time_mode, time, meas_mode));
            
            /* Select FSCAN Measurement */
            CHECKERR (rspr_ConfigureOperatingMode (io, RSPR_VAL_FREQ_MODE_FREQUENCY_SCAN));
            /* Initiate measurement */
            CHECKERR (rspr_InitiateWithStream (io, 10000, PORT));
            ResumeTimerCallbacks();
            SetWaitCursor(0);
            gRun = 1;

            break;
    }
    return 0;
}

int CVICALLBACK stopButton (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
    {
		case EVENT_COMMIT:
            CHECKERR (rspr_Abort (io));
            SuspendTimerCallbacks ();
            gRun = 0;
            SetCtrlAttribute(panelHandle,PANEL_START,ATTR_DIMMED,0);
			break;
    }
	return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
    {
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
            rspr_Abort (io);
            rspr_close (io);
			QuitUserInterface (0);
			break;
    }
	return 0;
}

int CVICALLBACK timerCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
    int i=0;
    TpSessionData pdata = NULL;
    ViStatus error = VI_SUCCESS;
	
    switch (event)
    {
		case EVENT_TIMER_TICK:
            checkErr (viGetAttribute (io, VI_ATTR_USER_DATA, &pdata));
            CHECKERR (rspr_ReadUDPFSCAN (io, RSPR_READ_BUFFER_SIZE, NULL, gLevel, NULL,
                                         NULL, NULL, gFreqLow, gFreqHigh));
            for (i = 0; i < RSPR_READ_BUFFER_SIZE; i++)
                gFrequency[i] = gFreqHigh[i] * UINT_MAX + gFreqLow[i];
            
            //DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
            PlotXY (panelHandle, PANEL_GRAPH, gFrequency, gLevel,
                    RSPR_READ_BUFFER_SIZE, VAL_DOUBLE, VAL_SHORT_INTEGER,
                    VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID, 1, VAL_RED);
			break;
    }
    
Error:
	return 0;
}

/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with error message.
/*           In case Device Clear is requested, it is necessary to set the
/*           argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{
    if (rspr_status < 0)
    {
        ViChar      error_string[1024];
        ViChar      error_message[256];
        ViStatus    temp_status;

        /* Query the error message from the instrument driver. This
         * function gets the error message for VISA and driver failures
         */
        temp_status = rspr_error_message(io, rspr_status, error_message);
        
        if (temp_status == VI_WARN_UNKNOWN_STATUS)
        {
            /* Query the error message from the instrument. This
             * function gets the error message directly from instrument, like
             * execution error, out of range error etc.
             */
            rspr_error_query(io, &temp_status, error_message);
        }
        sprintf (error_string, "Error 0x%8X occured:\n%s", rspr_status, error_message);
        MessagePopup ("ERROR", error_string);
    }
    
    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (io);
        viPrintf (io, "*CLS\n");
    }
    
    rspr_close (io);
    io = 0;
    gRun = VI_FALSE;
}

